﻿using Microsoft.Extensions.Logging;
using VegetableShop.API.Services;
using VegetableShop.Domain.Events;
using VegetableShop.Domain.Model;
using VegetableShop.Domain.Repositories;
using VegetableShop.Infrastructure.Persistence;

namespace VegetableShop.API
{
    public class Program
    {
        private readonly ConsumerServices _consumerServices;
        private readonly FarmerServices _farmerServices;

        private ILogger<Program> _logger;
        private ILoggerFactory _loggerFactory;

        private ILoggerFactory SetupLogging()
        {
            _loggerFactory = LoggerFactory.Create(builder =>
            {
                builder
                    .AddFilter("Microsoft", LogLevel.Warning)
                    .AddFilter("System", LogLevel.Warning)
                    .AddFilter("LoggingConsoleApp.Program", LogLevel.Debug)
                    .AddConsole();
            });
            _logger = _loggerFactory.CreateLogger<Program>();
            return _loggerFactory;
        }

        private Program()
        {
            var loggerFactory = SetupLogging();
            var context = new Context(loggerFactory);

            ConsumerRepository consumerRepository = new ConsumerRepositoryImpl(context);
            FarmerRepository farmerRepository = new FarmerRepositoryImpl(context);
            VegetableRepository vegetableRepository = new VegetableRepositoryImpl(context);

            _consumerServices = new ConsumerServices(loggerFactory,
                consumerRepository,
                farmerRepository,
                vegetableRepository);

            _farmerServices = new FarmerServices(loggerFactory,
                farmerRepository,
                vegetableRepository);

            DomainEvents.AddObserver(new LogItemOnFarmerPutOnSaleEvent(loggerFactory));
        }

        private void Run()
        {
            _logger.LogInformation("Calling _consumerServices.BuyVegetables");
            _farmerServices.PutOnSale(1, 1, new Price(1, "EUR"));
            _consumerServices.BuyVegetables(1, 1, 1, 10);
        }

        public static void Main()
        {
            new Program().Run();
        }
    }
}