using Microsoft.Extensions.Logging.Abstractions;
using VegetableShop.Domain.Model;
using Xunit;
using Xunit.Gherkin.Quick;

namespace VegetableShop.Domain.AcceptanceTests
{
    [FeatureFile("./Features/PutOnSale.feature")]
    public sealed class PutOnSale : Feature
    {
        private static readonly NullLoggerFactory LoggerFactory = new NullLoggerFactory();

        private Farmer _farmer;
        private Vegetable _vegetable;

        [Given(@"A farmer")]
        public void A_Farmer()
        {
            _farmer = new Farmer(LoggerFactory);
            Assert.NotNull(_farmer);
        }
        
        [And(@"A vegetable of name (\w+)")]
        public void A_Vegetable(string name)
        {
            _vegetable = new Vegetable(name);
        }
        
        [When(@"I put the vegetable on sale for (\d+) (\w+)")]
        public void I_Put_On_Sale(decimal value, string currency)
        {
            var price = new Price(value, currency);
            _farmer.PutOnSale(_vegetable, price);
            Assert.Contains(_farmer.VegetablesForSale, pair => pair.Key == _vegetable);
        }
        
        [Then(@"The vegetable (\w+) should be on sale at price (\d+) (\w+)")]
        public void The_vegetable_should_be_on_sale_at_price_x(string name, 
            decimal value,
            string currency)
        {
            Assert.Equal(_vegetable.Name, name);
            var price = _farmer.VegetablesForSale[_vegetable];
            Assert.Equal(price.Value, value);
            Assert.Equal(price.Currency, currency);
        }
    }
}