using System;
using System.Collections.Generic;
using Microsoft.Extensions.Logging;
using VegetableShop.Domain.Errors;
using VegetableShop.Domain.Events;
using VegetableShop.Domain.Specifications;

namespace VegetableShop.Domain.Model
{
    public class Farmer
    {
        private readonly ILogger<Farmer> _logger;
        public Dictionary<Vegetable, Price> VegetablesForSale { get; } = new Dictionary<Vegetable, Price>();
        private static readonly long Id = new Random().Next(int.MaxValue);

        public Farmer(ILoggerFactory loggerFactory) => _logger = loggerFactory.CreateLogger<Farmer>();
        public void PutOnSale(Vegetable vegetable, Price price)
        {
            if (AuthorizedSaleSpecification.IsSatisfiedBy(this, vegetable, price))
            {
                VegetablesForSale.Add(vegetable, price);
                _logger.LogInformation($"Add {vegetable} for price {price}");
                DomainEvents.Notify(new FarmerPutOnSaleEvent(Id, vegetable, price));
            }
            else
            {
                throw new UnauthorizedSaleException(this, vegetable);
            }
        }

        public void WithdrawFromSale() => throw new NotImplementedException();

        public void ChangePrice() => throw new NotImplementedException();

        public override string ToString() => $"Farmer[id={Id}]";
    }
}